use std::{io::Read, sync::mpsc::Receiver};

pub struct ByteReceiver {
    rx: Receiver<bytes::Bytes>,
    prev: Option<(bytes::Bytes, usize)>,
}

impl ByteReceiver {
    pub fn new(rx: Receiver<bytes::Bytes>) -> Self {
        Self { rx, prev: None }
    }
}

impl Read for ByteReceiver {
    fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
        if let Some(ref mut prev) = self.prev {
            let limit = std::cmp::min(prev.0.len() - prev.1, buf.len());
            buf[..limit].copy_from_slice(&prev.0[prev.1..prev.1 + limit]);
            if prev.1 + limit < prev.0.len() {
                prev.1 += limit;
            } else {
                self.prev = None
            }
            Ok(limit)
        } else if let Ok(bytes) = self.rx.recv() {
            let limit = std::cmp::min(bytes.len(), buf.len());
            buf[..limit].copy_from_slice(&bytes[..limit]);
            if buf.len() < bytes.len() {
                self.prev = Some((bytes, buf.len()));
            }
            Ok(limit)
        } else {
            Ok(0)
        }
    }
}
