use base64::{prelude::BASE64_STANDARD, Engine};
use bytes::{Buf, Bytes};
use serde::Serialize;

#[derive(Serialize)]
pub struct ZstdPayload {
    m: Option<()>,
    t: &'static str,
    zbase64: String,
}

pub fn encode_all(source: Bytes, level: i32) -> std::io::Result<Vec<u8>> {
    let mut result = Vec::<u8>::new();

    let mut encoder = zstd::Encoder::new(&mut result, level)?;
    encoder.set_pledged_src_size(Some(source.len() as u64))?;
    encoder.multithread(128)?;
    encoder.long_distance_matching(false)?;
    std::io::copy(&mut source.reader(), &mut encoder)?;
    encoder.finish()?;

    Ok(result)
}

impl ZstdPayload {
    pub fn new(b: Bytes) -> anyhow::Result<Self> {
        let zst = encode_all(b, 1)?;
        let b64 = BASE64_STANDARD.encode(zst);

        let p = Self {
            m: None,
            t: "buffer",
            zbase64: b64,
        };

        Ok(p)
    }
}
